# coding: utf-8
# Project：xbsTestTools
# File：format_execution.py
# Author：张炼武
# Date ：2025/5/15 14:15
# IDE：PyCharm
import subprocess
from xbsTestTools.operate_controller import XbsUiOperate

class FormatExecution:
    def __init__(self, dev):
        self.dev = dev
        self.xuo = XbsUiOperate(dev)

    def return_var(self,widget_info):
        """返回操作参数"""
        var_s = {
            "text": None,
            "index": None,
            "resourceId": None,
            "class_": None,
            "image_path": None,
            "threshold": 0.98,
            "value": "",
            "timeout": 3
        }
        if widget_info.get("text"):
            var_s["text"] = widget_info["text"]
        if widget_info.get("index"):
            var_s["index"] = widget_info["index"]
        if widget_info.get("resourceId"):
            var_s["resourceId"] = widget_info["resourceId"]
        if widget_info.get("class_"):
            var_s["class_"] = widget_info["class_"]
        if widget_info.get("image_path"):
            var_s["image_path"] = widget_info["image_path"]
        if widget_info.get("threshold"):
            var_s["threshold"] = widget_info["threshold"]
        if widget_info.get("value"):
            var_s["value"] = widget_info["value"]
        if widget_info.get("timeout"):
            var_s["timeout"] = widget_info["timeout"]

        return var_s
    def operation_analysis(self, widget_info):
        rv = self.return_var(widget_info)
        if widget_info.get("operate") == "click":
            # 点击
            return self.xuo.click(text=rv['text'], index=rv['index'], resourceId=rv['resourceId'], class_=rv['class_'], image_path=rv['image_path'], threshold=rv['threshold'])

        elif widget_info.get("operate") == "seek_click":
            # 滑动找到目标再点击
            return self.xuo.click_roll(text=rv['text'], index=rv['index'], resourceId=rv['resourceId'], class_=rv['class_'], image_path=rv['image_path'], threshold=rv['threshold'])
        elif widget_info.get("operate") == "set_text":
            # 输入文本
            return self.xuo.input(text=rv['text'], index=rv['index'], resourceId=rv['resourceId'], class_=rv['class_'], image_path=rv['image_path'], content=rv['value'],threshold=rv['threshold'])
        elif widget_info.get("operate") == "positioning":
            # 定位
            return self.xuo.exists(text=rv['text'], index=rv['index'], resourceId=rv['resourceId'], class_=rv['class_'], image_path=rv['image_path'], threshold=rv['threshold'],timeout=rv['timeout'])

    def actuator(self, widget_list, retry_count=3):
        """控件执行操作"""
        start_type = False
        for retry in range(1, retry_count + 1):
            print(f"执行ID: {retry}")
            subprocess.run(f"adb -s {self.dev} shell input keyevent 266")
            for index, widget in enumerate(widget_list):
                # 循环遍历执行前置操作
                if widget.get("preconditions"):
                    if not isinstance(widget.get("preconditions"), list):
                        widget.get("preconditions")()
                    else:
                        # 前置操作时一个list
                        for pre in widget.get("preconditions"):
                            self.operation_analysis(pre)
                result = self.operation_analysis(widget)
                # 后置操作
                if widget.get("post_operation"):
                    # 后置操作时一个方法的话执行方法
                    if not isinstance(widget.get("post_operation"), list):
                        widget.get("post_operation")()
                    else:
                        # 后置操作时一个list
                        for post_oper in widget.get("post_operation"):
                            self.operation_analysis(post_oper)
                if len(widget_list) == index + 1 and result:
                    start_type = True
                    break
            if start_type:
                break
        return start_type


# coding: utf-8
# Project：xbsUiOperate
# File：operate_controller.py
# Author：65
# Date ：2025/4/29 18:11
# IDE：PyCharm
import datetime
import os
from time import sleep
import uiautomator2 as u2
from xml.dom import minidom
import subprocess
import cv2
from xbsTestTools.adb_tools import AdbTools

class XbsUiTest:
    """UI控件信息查询"""

    def __init__(self, dev):
        self.dev = dev

    def get_xml(self):
        u2xml = self.get_u2_xml()
        if not u2xml:
            return self.get_shell_xml()
        return u2xml

    def get_u2_xml(self):
        """
        通过U2获取XML
        """
        try:
            try:
                d = u2.connect(self.dev)
            except:
                print(f"adb -s {self.dev} shell /data/local/tmp/atx-agent server --stop")
                subprocess.run(f"adb -s {self.dev} shell /data/local/tmp/atx-agent server --stop")  # 停止服务

                print(f"adb -s {self.dev} shell /data/local/tmp/atx-agent server -d")
                subprocess.run(f"adb -s {self.dev} shell /data/local/tmp/atx-agent server -d")  # 启动服务
                sleep(2)

                print(f"adb -s {self.dev} uninstall com.github.uiautomator.test")
                os.system(f"adb -s {self.dev} uninstall com.github.uiautomator.test")

                print(f"adb -s {self.dev} uninstall com.github.uiautomator")
                os.system(f"adb -s {self.dev} uninstall com.github.uiautomator")
                sleep(2)
                try:
                    d = u2.connect(self.dev)
                except:
                    print(f"{self.dev} U2 初始化失败")
                    return False
            xml = d.dump_hierarchy()
            return xml
        except Exception as e:
            # print(f"{self.dev} U2查找报错:{e}")
            return

    def get_shell_xml(self):
        """
        通过adb获取XML
        """
        dev = str(self.dev).replace("?", "").replace("？", "")
        os.popen(f"adb -s {self.dev} shell uiautomator dump /sdcard/{dev}_ui.xml")
        sleep(3)
        os.popen(f"adb -s {self.dev} pull /sdcard/{dev}_ui.xml")
        sleep(3)
        # 解析XML文件
        dom = minidom.parse(f'{dev}_ui.xml')
        raw_xml = dom.toprettyxml(indent='   ')
        # 去除多余空行
        formatted_xml = '\n'.join(
            line for line in raw_xml.split('\n') if line.strip()
        )
        return formatted_xml

    @staticmethod
    def return_dict(content):
        xpath_info = {
            # "index": "",
            "text": "",
            "resourceId": "",
            "class": "",
            "bounds": "",
            "checkable": "",
            "checked": "",
            "clickable": "",
            "enabled": "",
            "scrollable": ""
        }
        dict_ = content.strip().split(" ")
        for index, dx in enumerate(dict_):
            dx = str(dx.replace('"', ""))
            try:
                if "resource-id" in dx:
                    xpath_info['resourceId'] = dx[dx.find('=') + 1:]
                elif "text" in dx:
                    xpath_info['text'] = dx[dx.find('=') + 1:]
                elif "class" in dx:
                    xpath_info['class'] = dx[dx.find('=') + 1:]
                elif "checkable" in dx:
                    xpath_info['checkable'] = dx[dx.find('=') + 1:]
                elif "checked" in dx:
                    xpath_info['checked'] = dx[dx.find('=') + 1:]
                elif "clickable" in dx:
                    xpath_info['clickable'] = dx[dx.find('=') + 1:]
                elif "enabled" in dx:
                    xpath_info['enabled'] = dx[dx.find('=') + 1:]
                elif "scrollable" in dx:
                    xpath_info['scrollable'] = dx[dx.find('=') + 1:]
                elif "bounds" in dx:
                    coordinate = dx[dx.find('=') + 1:].split('][')
                    coordinate = [
                        i.replace("[", "").replace("]", "").replace("/>", "").replace("<", "").replace(">", "") for i in
                        coordinate]
                    x_ = coordinate[0].split(",")
                    y_ = coordinate[1].split(",")
                    x = int((int(x_[0]) + int(y_[0])) / 2)
                    y = int((int(x_[1]) + int(y_[1])) / 2)
                    xpath_info['bounds'] = [x, y]
            except:
                print("异常")
                print(dx)

        return xpath_info

    def get_dict_by_xml(self, text=None, index=None, resourceId=None, class_=None, scrollable=None):
        xml_content = self.get_xml()
        if not xml_content:
            return False
        xpath_list = []
        for content in xml_content.split("\n"):
            if scrollable:
                # 如果是寻找滑动元素则
                content_dict = self.return_dict(content)
                if content_dict['scrollable'] == 'true':
                    xpath_list.append(content_dict)
                continue

            content_dict = self.return_dict(content)
            if text and text != content_dict['text']:
                continue

            if resourceId and resourceId != content_dict['resourceId']:
                continue

            if class_ and class_ != content_dict['class']:
                continue
            xpath_list.append(content_dict)
        if not xpath_list:
            return False

        if index:
            try:
                return xpath_list[index]
            except:
                ...
        if scrollable:
            return xpath_list
        return xpath_list[0]


class XbsUiOperate:
    """执行UI操作"""
    def __init__(self, dev):
        self.dev = dev
        self.xut = XbsUiTest(dev)
        self.adb_tools = AdbTools(dev)

    def get_image_in_screenshot(self, small_image_path, threshold):
        """传入一张图片 获取在当前屏幕画面中的坐标位置"""
        ui_screenshot = self.screenshot(file="ui_screenshot", folder="current_screen")  # 获取当前画面信息
        # 读取截图和小图片
        screenshot = cv2.imread(ui_screenshot)  # cv2读取当前画面信息
        small_image = cv2.imread(small_image_path)  # cv2读取查找元素图片信息
        height, width, channels = small_image.shape  # 获取查找元素图片的高宽
        screenshot = cv2.cvtColor(screenshot, cv2.COLOR_BGR2GRAY)
        small_image = cv2.cvtColor(small_image, cv2.COLOR_BGR2GRAY)
        # 使用模板匹配
        result = cv2.matchTemplate(screenshot, small_image, cv2.TM_CCOEFF_NORMED)

        coordinate = []
        try:
            # 找到相似度最高的位置
            min_val, max_val, min_loc, max_loc = cv2.minMaxLoc(result)
            if max_val >= threshold:
                # 这里 max_loc 就是相似度最高的匹配位置
                coordinate.append(max_loc[0])
                coordinate.append(max_loc[1])
            coordinate[0] += width // 2  # 获取x轴中心
            coordinate[1] += height // 2  # 获取y轴中心
        except:
            return []
        return coordinate

    def click(self, text=None, index=None, resourceId=None, class_=None, image_path=None,threshold=0.98):
        if image_path:
            # 点击图片方式
            coordinate = self.get_image_in_screenshot(image_path,threshold)
            if not coordinate:
                return False
            os.system(f"adb -s {self.dev} shell input tap {coordinate[0]} {coordinate[1]}")
        else:
            # 点击控件方式
            widget_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId, class_=class_)
            if not widget_info:
                return False
            pos = widget_info['bounds']
            if not pos:
                return False
            os.system(f"adb -s {self.dev} shell input tap {pos[0]} {pos[1]}")
        return True

    def click_long(self, text=None, index=None, resourceId=None, class_=None, image_path=None,threshold=0.98):
        if image_path:
            # 点击图片方式
            coordinate = self.get_image_in_screenshot(image_path,threshold)
            if not coordinate:
                return False
            os.system(f"adb -s {self.dev} shell input tap {coordinate[0]} {coordinate[1]}")
        else:
            widget_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId, class_=class_)
            if not widget_info:
                return False
            pos = widget_info['bounds']
            if not pos:
                return False
            os.system(f"adb -s {self.dev} shell input swipe {pos[0]} {pos[1]} {pos[0]} {pos[1]} 2000")  # 长按2s
        return True

    def click_roll(self, text=None, index=None, resourceId=None, class_=None, image_path=None,threshold=0.98):
        """
        滑动找到元素再点击
        """
        if image_path:
            # 点击图片方式
            widget_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId, class_=class_,
                                                   scrollable=True)
            for widget in widget_info:
                for index in range(10):
                    os.system(
                        f"adb -s {self.dev} shell input swipe {widget['bounds'][0]} {widget['bounds'][1]} {widget['bounds'][0]} {int(widget['bounds'][1]) + 400}")
                for index in range(10):
                    os.system(
                        f"adb -s {self.dev} shell input swipe {widget['bounds'][0]} {widget['bounds'][1]} {widget['bounds'][0]} {int(widget['bounds'][1]) - 200}")
                    coordinate = self.get_image_in_screenshot(image_path,threshold)
                    if not coordinate:
                        continue

                    os.system(f"adb -s {self.dev} shell input tap {coordinate[0]} {coordinate[1]}")
                    return True
        else:
            widget_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId, class_=class_,
                                                   scrollable=True)
            for widget in widget_info:
                for index in range(10):
                    os.system(
                        f"adb -s {self.dev} shell input swipe {widget['bounds'][0]} {widget['bounds'][1]} {widget['bounds'][0]} {int(widget['bounds'][1]) + 400}")
                for index in range(10):
                    os.system(
                        f"adb -s {self.dev} shell input swipe {widget['bounds'][0]} {widget['bounds'][1]} {widget['bounds'][0]} {int(widget['bounds'][1]) - 200}")
                    widget_click_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId,
                                                                 class_=class_)
                    if not widget_click_info:
                        continue
                    pos = widget_click_info['bounds']
                    if not pos:
                        return False
                    os.system(f"adb -s {self.dev} shell input tap {pos[0]} {pos[1]}")
                    return True
        return False

    def exists(self, text=None, index=None, resourceId=None, class_=None, image_path=None, timeout=3,threshold=0.98):
        if image_path:
            for i in range(timeout):
                coordinate = self.get_image_in_screenshot(image_path,threshold)
                if coordinate:
                    return True
                sleep(1)
        else:
            for i in range(timeout):
                widget_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId, class_=class_)
                if widget_info:
                    return True
                sleep(1)
        return False

    def input(self, text=None, index=None, resourceId=None, class_=None, image_path=None, content=None, threshold=0.98):
        if not self.adb_tools.select_packages(package_name="com.android.adbkeyboard"):
            subprocess.run(
                f"adb -s {self.dev} install --bypass-low-target-sdk-block {os.path.dirname(os.path.abspath(__file__)) + os.sep}ADBKeyBoard.apk")
        subprocess.run(
            f"adb -s {self.dev} shell settings put secure enabled_input_methods com.android.adbkeyboard/.AdbIME")  # 激活输入法
        subprocess.run(f"adb -s {self.dev} shell ime set com.android.adbkeyboard/.AdbIME")  # 使用输入法

        if image_path:
            coordinate = self.get_image_in_screenshot(image_path, threshold)
            if not coordinate:
                return False
            subprocess.run(f"adb -s {self.dev} shell input tap {coordinate[0]} {coordinate[1]}")
        else:
            self.click(text=text, index=index, resourceId=resourceId, class_=class_)

        if self.click(text="继续"):
            if image_path:
                coordinate = self.get_image_in_screenshot(image_path, threshold)
                if not coordinate:
                    return False
                subprocess.run(f"adb -s {self.dev} shell input tap {coordinate[0]} {coordinate[1]}")
            else:
                self.click(text=text, index=index, resourceId=resourceId, class_=class_)

        subprocess.run(f"adb -s {self.dev} shell input text '{content}'")
        self.click(text="完成")

        return True

    def input_del(self, text=None, index=None, resourceId=None, class_=None):
        content = self.get_text(text=text, index=index, resourceId=resourceId, class_=class_)
        if not content:
            return True
        try:
            for n in range(len(content)):  # 根据文本的长度执行删除
                subprocess.run(f"adb -s {self.dev} shell input keyevent 67", shell=True, check=True)
            return True
        except:
            return False

    def roll(self, roll_type="down_to_top", ratio="middle", edge=False):
        size = self.get_size()  # 高  宽
        if not size:
            return False

        if ratio == "left":
            # 左边滑动
            x_start_t = int(int(size[1]) / 4)
            y_start_t = int(int(size[0]) / 3)
        elif ratio == "right":
            # 右边滑动
            x_start_t = int(int(size[1]) / 4) * 3
            y_start_t = int(int(size[0]) / 3)
        else:
            # 中间滑动
            x_start_t = int(int(size[1]) / 4) * 2
            y_start_t = int(int(size[0]) / 3)

        if roll_type == "down_to_top":
            # 从下往上滑动
            x_start = x_start_t
            x_end = x_start
            y_end = y_start_t
            y_start = y_start_t * 2
            os.system(f"adb -s {self.dev} shell input swipe {x_start} {y_start} {x_end} {y_end}")
        elif roll_type == "top_to_down":
            # 从上往下滑动
            x_start = x_start_t
            x_end = x_start
            y_end = y_start_t * 2
            y_start = y_start_t
            os.system(f"adb -s {self.dev} shell input swipe {x_start} {y_start} {x_end} {y_end}")
        elif roll_type == "left_to_right":
            # 从左往右滑动
            x_start = int(x_start_t - (x_start_t / 4))
            x_end = int(x_start_t + (x_start_t / 4))
            y_end = y_start_t + int(y_start_t / 4)
            y_start = y_start_t + int(y_start_t / 4)
            os.system(f"adb -s {self.dev} shell input swipe {x_start} {y_start} {x_end} {y_end}")
        elif roll_type == "right_to_left":
            # 从右往左滑动
            x_start = int(x_start_t + (x_start_t / 4))
            x_end = int(x_start_t - (x_start_t / 10))
            y_end = y_start_t + int(y_start_t / 4)
            y_start = y_start_t + int(y_start_t / 4)
            os.system(f"adb -s {self.dev} shell input swipe {x_start} {y_start} {x_end} {y_end}")

    def app_start(self, package_name):
        self.app_stop(package_name)  # 启动应用之前先关闭应用
        try:
            os.popen(f"adb -s {self.dev} shell monkey -p {package_name} -c android.intent.category.LAUNCHER 1")
        except Exception as e:
            return False
        return True

    def app_stop(self, package_name):
        os.system(f"adb -s {self.dev} shell am force-stop {package_name}")
        return True

    def install_app(self, apk_file):
        if not os.path.exists(apk_file):
            return False
        os.system(f"adb -s {self.dev} install {apk_file}")
        return True

    def uninstall_app(self, package_name):
        os.system(f"adb -s {self.dev} uninstall {package_name}")
        return True

    def screen(self):
        """唤醒屏幕"""
        os.system(f"adb -s {self.dev} shell input keyevent 266")
        return True

    def screen_no(self):
        """休眠"""
        os.system(f"adb -s {self.dev} shell input keyevent 266")
        os.system(f"adb -s {self.dev} shell input keyevent 26")
        return True

    def reboot(self):
        """重启"""
        os.system(f"adb -s {self.dev} reboot")
        return True

    def reboot_p(self):
        """关机"""
        os.system(f"adb -s {self.dev} reboot -p")
        return True

    def get_text(self, text=None, index=None, resourceId=None, class_=None):
        widget_info = self.xut.get_dict_by_xml(text=text, index=index, resourceId=resourceId, class_=class_)
        if not widget_info:
            return False
        return widget_info['text'].strip()

    def screenshot(self, file=None, folder="debug"):
        """截图"""
        root_path = os.path.dirname(os.path.abspath(__file__)) + os.sep
        current_folder = f"{root_path}screenshot_folder{os.sep}{folder}{os.sep}"
        if not os.path.exists(current_folder):
            os.makedirs(current_folder)  # 创建文件夹

        if not file:
            file = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

        try:
            os.system(f'adb -s {self.dev} exec-out screencap -p > "{current_folder}{file}.png"')
        except:
            return False
        return f"{current_folder}{file}.png"

    def screenrecord(self, file=None, time=30):
        """
        录制视频
        """
        try:
            if not file:
                file = datetime.datetime.now().strftime("%Y%m%d%H%M%S")

            os.system(f'adb -s {self.dev} shell screenrecord --time-limit {time} "/sdcard/{file}.mp4"')
            os.system(f'adb -s {self.dev} pull "/sdcard/{file}.mp4"')
        except:
            return False
        return f"{file}.mp4"
    def sound_recording(self):
        file = self.screenrecord()
        print(file)

    def get_size(self):
        try:
            result = os.popen("adb -s 03084S000080 shell wm size").read()
            size = result.replace("Physical size: ", "").strip().split("x")
            return size
        except:
            print(f"{self.dev} 获取屏幕分辨率失败")
            return False


if __name__ == '__main__':
    # dev = "03076S000002"
    # xuo = XbsUiOperate(dev)
    # print(xuo.screenshot())
    # xut = XbsUiTest(dev)
    # xml_content = xut.get_u2_xml()
    # print(xml_content)
    # print(xut.get_dict_by_xml(xml_content, text="系统"))
    ...



# coding: utf-8
# Project：xbsTestTools
# File：try_error.py
# Author：张炼武
# Date ：2025/5/15 11:12
# IDE：PyCharm
from functools import wraps

def try_catch(func):
    """
    异常捕获装饰类
    """
    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as e:
            print(f"执行方法 {func.__name__} 时发生错误: {str(e)}")
            return None
    return wrapper


# -*- coding: utf-8 -*-
import datetime
import functools
import inspect
import os
import re
import sys
import logging
from .colors import Fore as ForegroundColors
from .jsonlogger import JsonFormatter
from logging.handlers import TimedRotatingFileHandler, SysLogHandler,RotatingFileHandler
from logging import CRITICAL, ERROR, WARNING, WARN, INFO, DEBUG, NOTSET  # noqa: F401


try:
    import curses  # type: ignore
except ImportError:
    curses = None

__author__ = """Chris Hager"""
__email__ = 'chris@linuxuser.at'
__version__ = '1.7.0'

# Python 2+3 compatibility settings for logger
bytes_type = bytes
if sys.version_info >= (3,):
    unicode_type = str
    basestring_type = str
    xrange = range
else:
    # The names unicode and basestring don't exist in py3 so silence flake8.
    unicode_type = unicode  # noqa
    basestring_type = basestring  # noqa

# Formatter defaults
# 红色信息： %(end_color)s %(message)s
# 跟随等级颜色：%(color)s %(message)s
DEFAULT_FORMAT = '%(color)s[%(levelname)1.1s %(asctime)s %(pathname)s:%(lineno)d]%(color)s %(message)s'
DEFAULT_DATE_FORMAT = '%y%m%d %H:%M:%S'
DEFAULT_COLORS = {
    DEBUG: ForegroundColors.CYAN,
    INFO: ForegroundColors.GREEN,
    WARNING: ForegroundColors.YELLOW,
    ERROR: ForegroundColors.RED,
    CRITICAL: ForegroundColors.RED
}


# Attribute which all internal loggers carry
LOGZERO_INTERNAL_LOGGER_ATTR = "_is_logzero_internal"

# Logzero default logger
logger = None

# Current state of the internal logging settings
_loglevel = DEBUG
_logfile = None
_formatter = None


def setup_logger(name=__name__, logfile=None, level="DEBUG", maxBytes=0, backupCount=0, fileLoglevel=None, isRootLogger=False):

    if level == "DEBUG":
        level = DEBUG
    elif level == "WARNING":
        level = WARNING
    elif level == "ERROR":
        level = ERROR
    elif level == "WARN":
        level = WARN
    elif level == "NOTSET":
        level = NOTSET
    elif level == "CRITICAL":
        level = CRITICAL
    else:
        level = INFO

    _logger = logging.getLogger(None if isRootLogger else name)
    _logger.propagate = False

    _logger.setLevel(logging.DEBUG)

    # Setup default formatter
    _formatter = LogFormatter()

    # Reconfigure existing handlers
    stderr_stream_handler = None
    for handler in list(_logger.handlers):
        if hasattr(handler, LOGZERO_INTERNAL_LOGGER_ATTR):
            if isinstance(handler, logging.FileHandler):
                _logger.removeHandler(handler)
                continue
            elif isinstance(handler, logging.StreamHandler):
                stderr_stream_handler = handler

        handler.setLevel(level)
        handler.setFormatter(_formatter)


    stderr_stream_handler = logging.StreamHandler()
    setattr(stderr_stream_handler, LOGZERO_INTERNAL_LOGGER_ATTR, True)
    stderr_stream_handler.setLevel(level)
    stderr_stream_handler.setFormatter(_formatter)
    _logger.addHandler(stderr_stream_handler)

    if logfile:
        rotating_filehandler = RotatingFileHandler(filename=f"{logfile}{os.sep}{datetime.datetime.now().strftime('%Y%m%d')}.log", maxBytes=maxBytes, backupCount=backupCount)
        setattr(rotating_filehandler, LOGZERO_INTERNAL_LOGGER_ATTR, True)
        rotating_filehandler.setLevel(fileLoglevel or level)
        rotating_filehandler.setFormatter(_formatter)
        _logger.addHandler(rotating_filehandler)
    return _logger


class LogFormatter(logging.Formatter):
    """
    Log formatter used in Tornado. Key features of this formatter are:
    * Color support when logging to a terminal that supports it.
    * Timestamps on every log line.
    * Robust against str/bytes encoding problems.
    """

    def __init__(self,
                 color=True,
                 fmt=DEFAULT_FORMAT,
                 datefmt=DEFAULT_DATE_FORMAT,
                 colors=DEFAULT_COLORS):
        logging.Formatter.__init__(self, datefmt=datefmt)

        self._fmt = fmt
        self._colors = {}
        self._normal = ''

        if color and _stderr_supports_color():
            self._colors = colors
            self._normal = ForegroundColors.RESET

    def format(self, record):
        try:
            message = record.getMessage()
            assert isinstance(message,
                              basestring_type)  # guaranteed by logging
            # Encoding notes:  The logging module prefers to work with character
            # strings, but only enforces that log messages are instances of
            # basestring.  In python 2, non-ascii bytestri
            # ngs will make
            # their way through the logging framework until they blow up with
            # an unhelpful decoding error (with this formatter it happens
            # when we attach the prefix, but there are other opportunities for
            # exceptions further along in the framework).
            #
            # If a byte string makes it this far, convert it to unicode to
            # ensure it will make it out to the logs.  Use repr() as a fallback
            # to ensure that all byte strings can be converted successfully,
            # but don't do it by default so we don't add extra quotes to ascii
            # bytestrings.  This is a bit of a hacky place to do this, but
            # it's worth it since the encoding errors that would otherwise
            # result are so useless (and tornado is fond of using utf8-encoded
            # byte strings wherever possible).
            record.message = _safe_unicode(message)
        except Exception as e:
            record.message = "Bad message (%r): %r" % (e, record.__dict__)

        record.asctime = self.formatTime(record, self.datefmt)

        if record.levelno in self._colors:
            record.color = self._colors[record.levelno]
            record.end_color = self._normal
        else:
            record.color = record.end_color = ''

        formatted = self._fmt % record.__dict__

        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
        if record.exc_text:
            # exc_text contains multiple lines.  We need to _safe_unicode
            # each line separately so that non-utf8 bytes don't cause
            # all the newlines to turn into '\n'.
            lines = [formatted.rstrip()]
            lines.extend(
                _safe_unicode(ln) for ln in record.exc_text.split('\n'))
            formatted = '\n'.join(lines)
        return formatted.replace("\n", "\n    ")


def _stderr_supports_color():
    # Colors can be forced with an env variable
    if os.getenv('LOGZERO_FORCE_COLOR') == '1':
        return True

    # Windows supports colors with colorama
    if os.name == 'nt':
        return True

    # Detect color support of stderr with curses (Linux/macOS)
    if curses and hasattr(sys.stderr, 'isatty') and sys.stderr.isatty():
        try:
            curses.setupterm()
            if curses.tigetnum("colors") > 0:
                return True

        except Exception:
            pass

    return False


_TO_UNICODE_TYPES = (unicode_type, type(None))


def to_unicode(value):
    """
    Converts a string argument to a unicode string.
    If the argument is already a unicode string or None, it is returned
    unchanged.  Otherwise it must be a byte string and is decoded as utf8.
    """
    if isinstance(value, _TO_UNICODE_TYPES):
        return value
    if not isinstance(value, bytes):
        raise TypeError(
            "Expected bytes, unicode, or None; got %r" % type(value))
    return value.decode("utf-8")


def _safe_unicode(s):
    try:
        return to_unicode(s)
    except UnicodeDecodeError:
        return repr(s)


def __remove_internal_loggers(logger_to_update, disableStderrLogger=True):
    for handler in list(logger_to_update.handlers):
        if hasattr(handler, LOGZERO_INTERNAL_LOGGER_ATTR):
            if isinstance(handler, RotatingFileHandler):
                logger_to_update.removeHandler(handler)
            elif isinstance(handler, SysLogHandler):
                logger_to_update.removeHandler(handler)
            elif isinstance(handler, logging.StreamHandler) and disableStderrLogger:
                logger_to_update.removeHandler(handler)



def _get_json_formatter(json_ensure_ascii):
    supported_keys = [
        'asctime',
        'filename',
        'funcName',
        'levelname',
        'levelno',
        'lineno',
        'module',
        'message',
        'name',
        'pathname',
        'process',
        'processName',
        'threadName'
    ]

    def log_format(x):
        return ['%({0:s})s'.format(i) for i in x]

    custom_format = ' '.join(log_format(supported_keys))
    return JsonFormatter(custom_format, json_ensure_ascii=json_ensure_ascii)


def log(level="DEBUG",logfile=""):
    return setup_logger(level=level,logfile=logfile)
__all__ = ['log']


# coding: utf-8
# Project：xbsTestTools
# File：adb_tools.py
# Author：张炼武
# Date ：2025/5/15 11:23
# IDE：PyCharm
import subprocess
from time import sleep

class AdbTools:
    def __init__(self, dev):
        self.dev = dev

    def wifi_assertion_internet(self):
        """
        断言WiFi是否可以上网
        :return: 网络连接状态，True表示正常，False表示异常
        :rtype: bool
        """
        try:
            cmd = f"adb -s {self.dev} shell ping -c 3 www.baidu.com"
            response = subprocess.run(cmd, stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE, text=True, check=False)

            if response.returncode != 0 or response.stderr:
                print(f"{self.dev} 网络互联网异常: {response.stderr}")
                return False

            ping_data = response.stdout
            # 正则表达式提取丢包率
            import re
            match = re.search(r'(\d+)% packet loss', ping_data)
            if not match:
                print(f"{self.dev} 无法解析ping结果: {ping_data}")
                return False

            packet_loss = float(match.group(1))

            if packet_loss <= 20:
                print(f"{self.dev} 网络正常，丢包率: {packet_loss}%")
                return True
            elif packet_loss >= 80:
                print(f"{self.dev} 设备Ping 网络丢包较高: {packet_loss}%")
                return True
            elif packet_loss == 100:
                print(f"{self.dev} 设备Ping 网络丢包100%，网络不可上网")
                return False
            else:
                print(f"{self.dev} 设备Ping 网络丢包率: {packet_loss}%")
                return packet_loss < 50  # 根据丢包率决定返回值
        except Exception as e:
            print(f"{self.dev} 网络检测过程出错: {str(e)}")
            return False

    
    def assertion_dev_type(self, timeout_=60):
        """
        获取设备在线状态，默认超时时间60s，每2s检查一次
        :param timeout_: 超时时间
        :type timeout_: 超时时间
        :return: boole
        :rtype: boole
        """
        for t in range((timeout_ // 2)):
            response = subprocess.run("adb devices", stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            if response.stderr:
                print(f"获取ADB设备失败 重新获取中")
                continue
            adb_data = str(response.stdout).replace("List of devices attached", "").strip()

            for adb_info in adb_data.split("\n"):
                adb_info = adb_info.split("\t")
                if adb_info[0] == self.dev and adb_info[1] == "device":
                    return True
            sleep(2)

    @staticmethod
    def select_dev_list(conditional="ALL"):
        """
        获取连接的全部ADB设备，可根据连接方式条件获取（ALL/USB/IP）
        :param conditional: 条件
        :type conditional: 条件
        :return: list
        :rtype: list
        """
        for t in range(3):
            response = subprocess.run("adb devices", stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            if response.stderr:
                print(f"获取ADB设备失败 重新获取中")
                continue
            adb_data = str(response.stdout).replace("List of devices attached", "").strip()
            dev_list = []
            for adb_info in adb_data.split("\n"):
                adb_info = adb_info.split("\t")
                if conditional == "USB" and "." not in adb_info[0]:
                    dev_list.append({"dev": adb_info[0], "type": adb_info[1]})
                elif conditional == "IP" and "." in adb_info[0]:
                    dev_list.append({"dev": adb_info[0], "type": adb_info[1]})
                elif conditional == "ALL":
                    dev_list.append({"dev": adb_info[0], "type": adb_info[1]})
            print(dev_list)
            return dev_list

    
    def dow_screen(self):
        response = subprocess.run(f"adb -s {self.dev} shell dumpsys power", stdout=subprocess.PIPE,
                                  stderr=subprocess.PIPE, text=True)
        if not response.stderr:
            if "mWakefulness=Awake" not in response.stdout:
                subprocess.run(f"adb -s {self.dev} shell input keyevent KEYCODE_WAKEUP")
                print(f"{self.dev} 屏幕唤醒成功")
                return True
        print(f"{self.dev} 获取屏幕状态信息异常,屏幕唤醒失败")

    
    def assertion_dev_launcher(self, Launcher_name, timeout_=60):
        """
        获取设备 当前Launcher，默认超时时间60s，每2s检查一次
        :param Launcher_name:
        :type Launcher_name:
        :param timeout_: 超时时间
        :type timeout_: 超时时间
        :return: boole
        :rtype: boole
        """
        for t in range((timeout_ // 2)):
            response = subprocess.run(f"adb -s {self.dev} shell dumpsys window | grep mCurrentFocus",
                                      stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            if response.stderr:
                print(f"{self.dev} 获取ADB设备Launcher失败 重新获取中")
                sleep(2)
                continue
            launcher_info = str(response.stdout).strip()
            print(f"{self.dev} {launcher_info}")
            if Launcher_name in launcher_info:
                return True
            sleep(2)

    
    def assertion_wifi_type(self, type_=""):
        """
        获取WIFI开关状态，0为关闭 1为打开  写入条件时返回对应的结果  如果为空的话 则返回当前WIFI状态
        :param type_: 条件  1or2
        :type type_:
        :return: boole or str
        :rtype:
        """
        for t in range(3):
            response = subprocess.run(f"adb -s {self.dev} shell settings get global wifi_on", stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE, text=True)
            if response.stderr:
                print(f"{self.dev} 获取ADB设备WIFI开关状态失败 重新获取中")
                sleep(2)
                continue
            wifi_onoff_info = str(response.stdout).strip()
            print(f"{self.dev} WIFI 开关状态 {wifi_onoff_info}")
            if type_:
                if type_ in wifi_onoff_info:
                    return True
            else:
                return wifi_onoff_info
            break

    
    def onoff_wifi(self, type_=False):
        """
        WIFI开关  打开 or 关闭  默认关闭
        :param type_:
        :type type_:
        :return: boole
        :rtype: boole
        """
        for t in range(3):
            if type_:
                response = subprocess.run(f"adb -s {self.dev} shell cmd wifi set-wifi-enabled enabled")
                if response.stderr:
                    print(f"{self.dev} 打开WIFI开关失败 重新打开中")
                    sleep(2)
                    continue
                if self.assertion_wifi_type("1"):
                    print(f"{self.dev} WIFI开关打开")
                    return True
                print(f"{self.dev} WIFI开关打开失败")
            else:
                response = subprocess.run(f"adb -s {self.dev} shell cmd wifi set-wifi-enabled disabled")
                if response.stderr:
                    print(f"{self.dev} 关闭WIFI开关失败 重新关闭中")
                    sleep(2)
                    continue
                if self.assertion_wifi_type("0"):
                    print(f"{self.dev} WIFI开关 关闭")
                    return True
                print(f"{self.dev} WIFI开关关闭失败")
            break

    
    def select_wifi_type(self):
        """
        获取WIFI 状态
        :return: str
        :rtype: str
        """
        for t in range(3):
            response = subprocess.run(f"adb -s {self.dev} shell cmd wifi status", stdout=subprocess.PIPE,
                                      stderr=subprocess.PIPE, text=True)
            if response.stderr:
                print(f"{self.dev} WIFI状态信息获取失败，重新获取中")
                sleep(2)
                continue

            wifi_info = str(response.stdout)
            # print(f"{self.dev} {wifi_info}")

            if "Wifi is disabled" in wifi_info and "Wifi scanning is only available when wifi is enabled" in wifi_info:
                print(f"{self.dev} WIFI 未开启")
                return "is not enabled not connected"

            if "Wifi is not connected" in wifi_info:
                print(f"{self.dev} WIFI 已开启 未连接")
                return "is enabled not connected"

            wifi_info_data = wifi_info.split("\n")
            if "connected to" in wifi_info:
                wifi_name = wifi_info_data[3]
                print(f"{self.dev} WIFI 已开启 已连接 {wifi_name}")
                return "is enabled connected"

            break
        return "select error"

    
    def continuous_click(self, x, y, cn):
        """
        快速连点
        :param x: x轴
        :type x: str
        :param y: y轴
        :type y: str
        :param cn: 点击次数
        :type cn: int
        :return: True\False
        :rtype: boole
        """
        interval = 0.1
        for i in range(cn):
            subprocess.run(["adb", "-s", self.dev, "shell", "input", "tap", str(x), str(y)])
            sleep(interval)

    
    def select_packages(self, type_="ALL", package_name=None):
        """
        获取设备的所有应用包名
        :param type_: 类型  可以设置 3 表示未三方应用   或者 系统应用  默认为全部应用
        :type type_: str
        :param package_name: 查询指定的package
        :type package_name:str
        :return: 包名列表 or True\False
        :rtype: lsit boole
        """
        if type_ == "ALL":
            cmd = f"adb -s {self.dev} shell pm list packages"
        elif type_ == "三方":
            cmd = f"adb -s {self.dev} shell pm list packages -3"
        elif type_ == "系统":
            cmd = f"adb -s {self.dev} shell pm list packages -s"
        elif type_ == "当前":
            cmd = f"adb -s {self.dev} shell dumpsys window | grep mCurrentFocus"
        else:
            print(f"获取包名参数错误 {type_} 只能为一下分类['三方','系统','当前','ALL']")
            return

        response = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        if response.stderr:
            print(f"{self.dev} WIFI状态信息获取失败，重新获取中")
            return
        package_data = str(response.stdout).strip().replace("package:", "")

        # 查询是否存在指定应用包名
        if package_name:
            if package_name not in package_data:
                return
            return True

        # 查询当前运行的应用包名
        if type_ != "当前":
            package = package_data[package_data.find("u0") + 2:package_data.find("/")].strip()
            print(package)
            return package

        # 查询指定类型的包名list
        package_list = package_data.split("\n")
        print(f"[{len(package_list)}个] - {package_list}")
        return package_list

    
    def select_size(self):
        """
        获取设备的分辨率
        :return: int
        :rtype:
        """
        result = subprocess.run(['adb', '-s', self.dev, 'shell', 'wm', 'size'], capture_output=True, text=True)
        if result.returncode != 0:
            print(f"{self.dev} 获取设备分辨率失败")
            return

        size = result.stdout.strip().split('size:')[-1].split('x')
        width = int(size[1])
        height = int(size[0])
        return width, height

    
    def swipe(self, start_x, start_y, end_x, end_y, duration_ms=300):
        """
        滑动  可按照像素点来  也可以按照百分比来
        :param start_x: 开始位置X
        :type start_x: int/str
        :param start_y: 开始位置Y
        :type start_y: int/str
        :param end_x: 结束位置x
        :type end_x:int/str
        :param end_y:结束位置Y
        :type end_y:int/str
        :param duration_ms: 执行完成速度
        :type duration_ms: int
        :return:
        :rtype: boole
        """
        width, height = self.select_size()
        if not width or not height:
            print(f"{self.dev}滑动失败")
            return

        if "%" in str(start_x) and "%" in str(start_y) and "%" in str(end_x) and "%" in str(end_y):
            # 计算百分比坐标
            print(f"{self.dev} 滑动百分比 {start_x} {start_y} {end_x} {end_y}")
            start_x = int(width * int(start_x.replace('%', '')) / 100)
            start_y = int(height * int(start_y.replace('%', '')) / 100)
            end_x = int(width * int(end_x.replace('%', '')) / 100)
            end_y = int(height * int(end_y.replace('%', '')) / 100)

        command = f"adb -s {self.dev} shell input swipe {start_x} {start_y} {end_x} {end_y} {duration_ms}"
        subprocess.run(command)
        print(f"{self.dev} 滑动成功 {start_x} {start_y} {end_x} {end_y} {duration_ms}")
        return True

    
    def click(self, start_x, end_x, duration_ms=300):
        """
        点击  可按照像素点来  也可以按照百分比来
        :param start_x: 开始位置X
        :type start_x: int/str
        :param end_x: 结束位置x
        :type end_x:int/str
        :param duration_ms: 执行完成速度
        :type duration_ms: int
        :return:
        :rtype: boole
        """
        width, height = self.select_size()
        if not width or not height:
            print(f"{self.dev}滑动失败")
            return

        if "%" in str(start_x) and "%" in str(end_x):
            # 计算百分比坐标
            print(f"{self.dev} 点击百分比 {start_x} {end_x}")
            start_x = int(width * int(start_x.replace('%', '')) / 100)
            end_x = int(width * int(end_x.replace('%', '')) / 100)

        command = f"adb -s {self.dev} shell input tap {start_x} {end_x} {duration_ms}"
        subprocess.run(command)
        print(f"{self.dev} 点击成功 {start_x} {end_x} {duration_ms}")
        return True

    
    def reboot(self):
        """
        重启
        :return:
        :rtype:
        """
        subprocess.run(f"adb -s {self.dev} shell reboot")

    
    def shutdown(self):
        """
        关机
        :return:
        :rtype:
        """
        subprocess.run(f"adb -s {self.dev} shell reboot -p")




# coding: utf-8
# Project：xbsTestTools
# File：__init__.py.py
# Author：张炼武
# Date ：2025/5/15 16:41
# IDE：PyCharm

from .adb_tools import AdbTools
from .format_execution import FormatExecution
from .method_executor import MethodExecutor
from .operate_controller import XbsUiOperate,XbsUiTest
from .try_error import try_catch
from .log import log

__all__ = ['AdbTools','FormatExecution','MethodExecutor','XbsUiOperate','XbsUiTest','try_catch','log']


# coding: utf-8
# Project：xbsTestTools
# File：method_executor.py
# Author：张炼武
# Date ：2025/5/14 19:22
# IDE：PyCharm
import time
from typing import List, Callable, Any, Dict

class MethodExecutor:
    """
    方法执行器，用于按顺序执行多个方法
    """
    def __init__(self):
        self.methods: List[Dict[str, Any]] = []
        self.current_index = 0

    def add_method(self, method: Callable, *args, **kwargs) -> 'MethodExecutor':
        """
        添加要执行的方法
        :param method: 要执行的方法
        :param args: 方法的参数
        :param kwargs: 方法的关键字参数
        :return: 方法执行器实例
        """
        self.methods.append({
            'method': method,
            'args': args,
            'kwargs': kwargs
        })
        return self

    def execute(self, stop_on_error: bool = True) -> bool:
        """
        按顺序执行所有方法
        :param stop_on_error: 是否在遇到错误时停止执行
        :return: 是否所有方法都执行成功
        """
        success = True
        for i, method_info in enumerate(self.methods):
            method = method_info['method']
            args = method_info['args']
            kwargs = method_info['kwargs']

            try:
                print(f"开始执行第 {i + 1} 个方法: {method.__name__}")
                start_time = time.time()

                result = method(*args, **kwargs)

                end_time = time.time()
                execution_time = round(end_time - start_time, 2)

                print(f"方法 {method.__name__} 执行完成，耗时: {execution_time}秒")

                if not result:
                    print(f"方法 {method.__name__} 执行失败")
                    success = False
                    if stop_on_error:
                        # 执行过程失败停止
                        break

            except Exception as e:
                print(f"执行方法 {method.__name__} 时发生错误: {str(e)}")
                success = False
                if stop_on_error:
                    break

        return success

    def clear(self):
        """
        清空所有待执行的方法
        """
        self.methods.clear()
        self.current_index = 0


def for_run(v,fun):
    if type(v) != list and type(v) != int:
        print("传入的参数有误 只能是int或list")
        return False

    if v == int:
        for index in range(v):
            result = fun(index)
    else:
        for index in v:
            fun(index)

# 使用示例
if __name__ == "__main__":
    def method1(bb):
        print(f"执行方法1 等待4s {bb}")
        time.sleep(4)
        print("执行方法1 等待4s 完成")
        return True


    def method2():
        print("执行方法2 开始")
        return True


    def method3():
        print("执行方法3")
        print("执行方法3 等待2s")
        time.sleep(2)
        print("执行方法3 等待2s 完成")

        return True


    # 创建执行器实例
    executor = MethodExecutor()

    # 添加要执行的方法
    executor.add_method(method1,"111") \
        .add_method(method2) \
        .add_method(method3)

    # 执行所有方法
    success = executor.execute(stop_on_error=True)
    print(f"执行结果: {'成功' if success else '失败'}")
